/***************************************************************************
 *   copyright       : (C) 2009-2010 by Pascal Brachet                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "algobox.h"
#include "browserdialog.h"
#include "variabledialog.h"
#include "liredialog.h"
#include "afficherdialog.h"
#include "messagedialog.h"
#include "affectationdialog.h"
#include "conditiondialog.h"
#include "pourdialog.h"
#include "tantquedialog.h"
#include "aproposdialog.h"

#if defined( Q_WS_X11 )
#include "x11fontdialog.h"
#endif

#include <QSettings>
#include <QAction>
#include <QStatusBar>
#include <QMessageBox>
#include <QApplication>
#include <QDir>
#include <QDesktopWidget>
#include <QStyleFactory>
#include <QStyle>
#include <QDomDocument>
#include <QDomElement>
#include <QDomProcessingInstruction>
#include <QFile>
#include <QFileDialog>
#include <QFileInfo>
#include <QFontDatabase>
#include <QDebug>
#include <QTextStream>
#include <QTextCodec>
#include <QFont>
#include <QColor>
#include <QDesktopServices>
#include <QUrl>
#include <QStyleFactory>
#include <QStyle>
#include <QTextDocument>
#include <QPrintDialog>
#include <QPrinter>
#include <QDate>
#include <QStatusBar>

MainWindow::MainWindow(QWidget *parent)
    :QMainWindow( parent)
{
ui.setupUi(this);
QStatusBar * status=statusBar();
status->addPermanentWidget(new QLabel("AlgoBox "ALGOBOXVERSION),0);
LireConfig();
QFile file(":/documents/styleapplication.qss");
file.open(QFile::ReadOnly);
QString styleSheet = QLatin1String(file.readAll());
qApp->setStyleSheet(styleSheet);

#ifdef Q_WS_MACX
setWindowIcon(QIcon(":/images/algobox128.png"));
#else
setWindowIcon(QIcon(":/images/algobox22.png"));
#endif
connect(ui.pushButtonAjouterLigne, SIGNAL(clicked()), this, SLOT(AjouterLigne()));
connect(ui.pushButtonSupprimerLigne, SIGNAL(clicked()), this, SLOT(SupprimerLigne()));
connect(ui.pushButtonModifierLigne, SIGNAL(clicked()), this, SLOT(ModifierLigne()));

connect(ui.treeWidget, SIGNAL(currentItemChanged(QTreeWidgetItem *, QTreeWidgetItem *)),this,SLOT(ActualiserArbre()));

connect(ui.pushButtonVariable, SIGNAL(clicked()), this, SLOT(AjouterVariable()));

connect(ui.pushButtonLire, SIGNAL(clicked()), this, SLOT(AjouterLire()));

connect(ui.pushButtonAfficher, SIGNAL(clicked()), this, SLOT(AjouterAfficher()));

connect(ui.pushButtonMessage, SIGNAL(clicked()), this, SLOT(AjouterMessage()));

connect(ui.pushButtonAffectation, SIGNAL(clicked()), this, SLOT(AjouterAffectation()));

connect(ui.pushButtonCondition, SIGNAL(clicked()), this, SLOT(AjouterCondition()));

connect(ui.pushButtonBoucle, SIGNAL(clicked()), this, SLOT(AjouterBoucle()));

connect(ui.pushButtonTantque, SIGNAL(clicked()), this, SLOT(AjouterTantque()));

connect(ui.pushButtonExecuter, SIGNAL(clicked()), this, SLOT(JavascriptExport()));

fichierMenu = menuBar()->addMenu(QString::fromUtf8("Fichier"));
QAction *Act;

Act = new QAction(QIcon(":/images/nouveau.png"),QString::fromUtf8("Nouveau"), this);
Act->setShortcut(Qt::CTRL+Qt::Key_N);
connect(Act, SIGNAL(triggered()), this, SLOT(NouvelAlgo()));
fichierMenu->addAction(Act);

Act = new QAction(QIcon(":/images/ouvrir.png"),QString::fromUtf8("Ouvrir"), this);
Act->setShortcut(Qt::CTRL+Qt::Key_O);
connect(Act, SIGNAL(triggered()), this, SLOT(ChargerAlgo()));
fichierMenu->addAction(Act);

recentMenu=fichierMenu->addMenu(QString::fromUtf8("Récemment ouverts"));
for (int i = 0; i < 5; ++i) 
	{
	recentFileActs[i] = new QAction(this);
	recentFileActs[i]->setVisible(false);
	connect(recentFileActs[i], SIGNAL(triggered()),this, SLOT(fileOpenRecent()));
	recentMenu->addAction(recentFileActs[i]);
	}

Act = new QAction(QIcon(":/images/sauver.png"),QString::fromUtf8("Sauver"), this);
Act->setShortcut(Qt::CTRL+Qt::Key_S);
connect(Act, SIGNAL(triggered()), this, SLOT(SauverAlgo()));
fichierMenu->addAction(Act);

Act = new QAction(QIcon(":/images/sauversous.png"),QString::fromUtf8("Sauver Sous"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(SauverSousAlgo()));
fichierMenu->addAction(Act);

fichierMenu->addSeparator();

Act = new QAction(QString::fromUtf8("Exporter algorithme complet vers page web"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(ExporterVersHtml()));
fichierMenu->addAction(Act);

Act = new QAction(QString::fromUtf8("Exporter code vers fichier texte"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(ExporterVersTexte()));
fichierMenu->addAction(Act);

Act = new QAction(QString::fromUtf8("Exporter code vers document LaTeX"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(ExporterVersLatex()));
fichierMenu->addAction(Act);

Act = new QAction(QIcon(":/images/imprimer.png"),QString::fromUtf8("Imprimer algorithme"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(Imprimer()));
fichierMenu->addAction(Act);

fichierMenu->addSeparator();

Act = new QAction(QString::fromUtf8("Ouvrir un exemple"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(ChargerExemple()));
fichierMenu->addAction(Act);

fichierMenu->addSeparator();

#if defined( Q_WS_X11 )
Act = new QAction(QString::fromUtf8("Changer Police Interface"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(SetInterfaceFont()));
fichierMenu->addAction(Act);
fichierMenu->addSeparator();
#endif

Act = new QAction( QString::fromUtf8("Quitter"), this);
Act->setShortcut(Qt::CTRL+Qt::Key_Q);
connect(Act, SIGNAL(triggered()), this, SLOT(Quitter()));
fichierMenu->addAction(Act);

aideMenu = menuBar()->addMenu(QString::fromUtf8("Aide"));
Act = new QAction(QIcon(":/images/aide.png"),QString::fromUtf8("Aide"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(Aide()));
aideMenu->addAction(Act);
aideMenu->addSeparator();
Act = new QAction(QString::fromUtf8("A propos d'AlgoBox"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(APropos()));
aideMenu->addAction(Act);

fileToolBar = addToolBar("Fichier");
fileToolBar->setObjectName("Fichier");
fileToolBar->setIconSize(QSize(22,22 ));
fileToolBar->setFloatable(false);
fileToolBar->setOrientation(Qt::Horizontal);
fileToolBar->setMovable(false);
fileToolBar->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);

Act = new QAction(QIcon(":/images/nouveau22.png"), QString::fromUtf8("Nouveau"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(NouvelAlgo()));
fileToolBar->addAction(Act);

Act = new QAction(QIcon(":/images/ouvrir22.png"),QString::fromUtf8("Ouvrir"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(ChargerAlgo()));
fileToolBar->addAction(Act);

Act = new QAction(QIcon(":/images/sauver22.png"), QString::fromUtf8("Sauver"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(SauverAlgo()));
fileToolBar->addAction(Act);

Act = new QAction(QIcon(":/images/executer22.png"), QString::fromUtf8("Tester"), this);
connect(Act, SIGNAL(triggered()), this, SLOT(JavascriptExport()));
fileToolBar->addAction(Act);

UpdateRecentFile();

QFile interditsfile(":/documents/interdits.txt");
if (interditsfile.open(QFile::ReadOnly))
    {
    QString line;
    while (!interditsfile.atEnd()) 
	    {
	    line = interditsfile.readLine();
	    if (!line.isEmpty()) ListeNomsInterdits.append(line.trimmed());
	    }
    }

estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("AlgoBox : ")+nomFichier);
Init();
ActualiserArbre();
connect(ui.textEditDescription, SIGNAL(textChanged()),this,SLOT(ActualiserStatut()));
restoreState(windowstate, 0);
}

MainWindow::~MainWindow(){
}

void MainWindow::closeEvent(QCloseEvent *e)
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("AlgoBox : "),
					QString::fromUtf8("L'algorithme courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverAlgo();
		break;
		case 1:
		break;
		case 2:
		default:
		e->ignore();
		return;
		break;
		}
	}
SauverConfig();
e->accept();
}

void MainWindow::Quitter()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("AlgoBox : "),
					QString::fromUtf8("L'algorithme courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverAlgo();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
SauverConfig();
qApp->quit();
}
//*************************
void MainWindow::AjouterLigne()
{
QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
NouvelleLigne(curItem);
ActualiserArbre();
ActualiserStatut();
}
void MainWindow::NouvelleLigne(QTreeWidgetItem *item)
{
if (!item) return;
QTreeWidgetItem *newItem = 0;
QTreeWidgetItem *nextItem = 0;
if (item->text(0)=="DEBUT_ALGORITHME")
    {
    newItem=new QTreeWidgetItem(QStringList(QString("")));
    newItem->setData(0,Qt::UserRole,QString("103#commentaire"));
    debutItem->insertChild(0,newItem);
    ui.treeWidget->setCurrentItem(newItem);
    }
else if ((item->text(0).left(3)=="FIN") ||(item->text(0)=="SINON") )
    {
    if (item->parent())
	{
	int idx = item->parent()->childCount() - 1;
	nextItem = item->parent()->child(idx);
	NouvelleLigne(item->parent());
	}
    }
else if (item->text(0).left(5)=="DEBUT")
    {
    newItem=new QTreeWidgetItem(QStringList(QString("")));
    newItem->setData(0,Qt::UserRole,QString("103#commentaire"));
    if (item->parent())
	{
	item->parent()->insertChild(1,newItem);
	ui.treeWidget->setCurrentItem(newItem);	
	}
    }
else
    {
    if (item->parent()) newItem = new QTreeWidgetItem(item->parent(), item);
    newItem->setText(0,"");
    newItem->setData(0,Qt::UserRole,QString("103#commentaire"));
    ui.treeWidget->setCurrentItem(newItem);
    }
}
void MainWindow::SupprimerLigne()
{
QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
if (!curItem) return;
QString code=curItem->data(0,Qt::UserRole).toString();
QStringList parametres=code.split("#");
int type_para=parametres.at(0).toInt();
int query=1;
if (type_para==1)
    {
    query =QMessageBox::warning(this, "AlgoBox", QString::fromUtf8("Etes-vous sûr de vouloir supprimer la déclaration de cette variable?\n(si elle est utilisée dans le reste du code, l'algorithme ne pourra plus fonctionner)"),QString::fromUtf8("Confirmer la suppression"), QString::fromUtf8("Abandonner") );
    }
else if ((type_para==6) || (type_para==12) || (type_para==15)) 
    {
    query =QMessageBox::warning(this, "AlgoBox", QString::fromUtf8("La suppression de cette ligne entrainera la suppression de tout le bloc qui en dépend.\nEtes-vous sûr de vouloir supprimer tout ce bloc d'instructions?"),QString::fromUtf8("Confirmer la suppression"), QString::fromUtf8("Abandonner") );
    }
else
    {
    query =QMessageBox::warning(this, "AlgoBox", QString::fromUtf8("Etes-vous sûr de vouloir supprimer cette ligne?"),QString::fromUtf8("Confirmer la suppression"), QString::fromUtf8("Abandonner") );
    }
if (query==1) return;
QTreeWidgetItem *nextCurrent = 0;
if (curItem->parent()) 
	{
	int idx = curItem->parent()->indexOfChild(curItem);
	if (idx == curItem->parent()->childCount() - 1) idx--;
	else idx++;
	if (idx < 0) nextCurrent = curItem->parent();
	else nextCurrent = curItem->parent()->child(idx);
	} 
else 
	{
	int idx = ui.treeWidget->indexOfTopLevelItem(curItem);
	if (idx == ui.treeWidget->topLevelItemCount() - 1) idx--;
	else idx++;
	if (idx >= 0) nextCurrent = ui.treeWidget->topLevelItem(idx);
	}
delete curItem;
if (nextCurrent) ui.treeWidget->setCurrentItem(nextCurrent);
ActualiserArbre();
ActualiserStatut();
}

void MainWindow::ModifierLigne()
{
QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
if (!curItem) return;
QString code=curItem->data(0,Qt::UserRole).toString();
QStringList parametres=code.split("#");
int type_para=parametres.at(0).toInt();
if (type_para==2) ModifierLire();
else if (type_para==3) ModifierAfficher();
else if (type_para==4) ModifierMessage();
else if (type_para==5) ModifierAffectation();
else if (type_para==6) ModifierCondition();
else if (type_para==12) ModifierBoucle();
else if (type_para==15) ModifierTantque();
ActualiserArbre();
ActualiserStatut();
}

void MainWindow::ActualiserArbre()
{
bool canAddline = true;
bool canRemoveline = true;
bool canModifyline=true;
QTreeWidgetItem *current = ui.treeWidget->currentItem();
QTreeWidgetItem *next = 0;
if (current) 
	{
	QString code=current->data(0,Qt::UserRole).toString();
	QStringList parametres=code.split("#");
	int type_para=parametres.at(0).toInt();
	switch (type_para)
	    {
	    case 1: //VARIABLES
		    if (parametres.count()==3)
			{
			canRemoveline=true;
			canAddline = false;
			canModifyline=false;
			}
		    break;
	    case 2: //LIRE
		    if (parametres.count()==3)
			{
			canRemoveline=true;
			canAddline = true;
			canModifyline=true;
			}
		    break;
	    case 3: //AFFICHER
		    if (parametres.count()==4)
			{
			canRemoveline=true;
			canAddline = true;
			canModifyline=true;
			}
		    break;
	    case 4: //MESSAGE
		    if (parametres.count()==3)
			{
			canRemoveline=true;
			canAddline = true;
			canModifyline=true;
			}
		    break;
	    case 5: //AFFECTATION
		    if (parametres.count()==4)
			{
			canRemoveline=true;
			canAddline = true;
			canModifyline=true;
			}
		    break;
	    case 6: //SI
		    if (parametres.count()==2)
			{
			canRemoveline=true;
			canAddline = false;
			canModifyline=true;
			}
		    break;
	    case 7: //DEBUT_SI
		    if (parametres.count()==2)
			{
			canRemoveline=false;
			canAddline = true;
			canModifyline=false;
			}
		    break;
	    case 8: //FIN_SI
		    if (parametres.count()==2)
			{
			canRemoveline=false;
			canAddline = false;
			canModifyline=false;
			if (current->parent())
			    {
			    int idx = current->parent()->indexOfChild(current);
			    if (idx<current->parent()->childCount() - 1)
				{
				idx++;
				if (idx>=0) 
				    {
				    next=current->parent()->child(idx);
				    if (next->text(0)!="SINON") canAddline = true;			  
				    }
				}
			    else canAddline = true;
			    }
			}
		    break;
	    case 9: //SINON
		    if (parametres.count()==2)
			{
			canRemoveline=true;
			canAddline = false;
			canModifyline=false;
			}
		    break;
	    case 10: //DEBUT_SINON
		    if (parametres.count()==2)
			{
			canRemoveline=false;
			canAddline = true;
			canModifyline=false;
			}
		    break;
	    case 11: //FIN_SINON
		    if (parametres.count()==2)
			{
			canRemoveline=false;
			canAddline = true;
			canModifyline=false;
			}
		    break;
	    case 12: //POUR
		    if (parametres.count()==4)
			{
			canRemoveline=true;
			canAddline = false;
			canModifyline=true;
			}
		    break;
	    case 13: //DEBUT_POUR
		    if (parametres.count()==2)
			{
			canRemoveline=false;
			canAddline = true;
			canModifyline=false;
			}
		    break;
	    case 14: //FIN_POUR
		    if (parametres.count()==2)
			{
			canRemoveline=false;
			canAddline = true;
			canModifyline=false;
			}
		    break;
	    case 15: //TANT_QUE
		    if (parametres.count()==2)
			{
			canRemoveline=true;
			canAddline = false;
			canModifyline=true;
			}
		    break;
	    case 16: //DEBUT_TANT_QUE
		    if (parametres.count()==2)
			{
			canRemoveline=false;
			canAddline = true;
			canModifyline=false;
			}
		    break;
	    case 17: //FIN_TANT_QUE
		    if (parametres.count()==2)
			{
			canRemoveline=false;
			canAddline = true;
			canModifyline=false;
			}
		    break;
	    case 100: //DECLARATIONS VARIABLES
		    if (parametres.count()==2)
			{
			canRemoveline=false;
			canAddline = false;
			canModifyline=false;
			}
		    break;
	    case 101: //DEBUT_ALGO
		    if (parametres.count()==2)
			{
			canRemoveline=false;
			canAddline = true;
			canModifyline=false;
			}
		    break;
	    case 102: //FIN_ALGO
		    if (parametres.count()==2)
			{
			canRemoveline=false;
			canAddline = false;
			canModifyline=false;
			}
		    break;
	    case 103: //autres
	    default:
		    if (parametres.count()==2)
			{
			canRemoveline=true;
			canAddline = true;
			canModifyline=false;
			}
		    break;
	    }
	if (current->text(0).isEmpty()) ActiverBoutons();
	else DesactiverBoutons();
	}
else 
	{
	canAddline = false;
	canModifyline=false;
	DesactiverBoutons();
	}
ui.pushButtonAjouterLigne->setEnabled(canAddline);
ui.pushButtonSupprimerLigne->setEnabled(canRemoveline);
ui.pushButtonModifierLigne->setEnabled(canModifyline);
ui.treeWidget->setFocus();
ActualiserVariables();
ui.treeWidget->header()->resizeSections(QHeaderView::ResizeToContents);
}
void MainWindow::ActualiserVariables()
{
ListeNomsVariables.clear();
ListeTypesVariables.clear();
QString code;
QRegExp rxvar("(.*) EST_DU_TYPE (.*)");
for (int i = 0; i < variablesItem->childCount(); i++) 
    {
    code=variablesItem->child(i)->text(0);
    if (rxvar.indexIn(code)>-1)
	{
	ListeNomsVariables.append(rxvar.cap(1));
	ListeTypesVariables.append(rxvar.cap(2));
	}
    }
}
//*************************
void MainWindow::AjouterVariable()
{
QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
VariableDialog *variableDialog = new VariableDialog(this);
if (variableDialog->exec())
    {
    QString nomvariable=variableDialog->ui.lineEditVariable->text();
    nomvariable=FiltreNomVariable(nomvariable);
    QString type=variableDialog->ui.comboBoxVariable->currentText();
    QString code="";
    if (NomInterdit(nomvariable))
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Nom de variable interdit"));
	}
    else
	{
	QTreeWidgetItem *newItem =new QTreeWidgetItem(variablesItem);
	newItem->setText(0,nomvariable+" EST_DU_TYPE "+type);
	code="1#"+type+"#"+nomvariable;
	newItem->setData(0,Qt::UserRole,QString(code));
	if ((curItem) && (curItem!=variablesItem)) ui.treeWidget->setCurrentItem(curItem);
	else ui.treeWidget->setCurrentItem(debutItem);
	ActualiserStatut();
	}
    }
InitVariable();
ActualiserArbre();
}
void MainWindow::AjouterLire()
{
QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
if (!curItem || !curItem->parent()) return;
if (ListeNomsVariables.isEmpty())
    {
    QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Aucune variable n'a encore été définie."));
    return;
    }
LireDialog *lireDialog = new LireDialog(this,ListeNomsVariables.join("#"),ListeTypesVariables.join("#"));
if (lireDialog->exec())
    {
    QString nomvariable=lireDialog->ui.comboBoxLire->currentText();
    QString rang=lireDialog->ui.lineEditLire->text();
    QString code,type;
    code="";
    if (!nomvariable.isEmpty())
	{
	int i=ListeNomsVariables.indexOf(nomvariable);
	if (i>-1)
	    {
	    type=ListeTypesVariables.at(i);
	    if ((type=="LISTE") && (!rang.isEmpty())) 
		{
		curItem->setText(0,QString::fromUtf8("LIRE ")+nomvariable+"["+rang+"]");
		code="2#"+nomvariable+"#"+rang;
		}
	    else 
		{
		curItem->setText(0,QString::fromUtf8("LIRE ")+nomvariable);
		code="2#"+nomvariable+"#pasliste";
		}
	    curItem->setData(0,Qt::UserRole,QString(code));
	    }
	ui.treeWidget->setCurrentItem(curItem);
	ActualiserStatut();
	}
    else
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Pas de variable définie"));
	}
    }
InitLire();
ActualiserArbre();
}

void MainWindow::AjouterAfficher()
{
QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
if (!curItem || !curItem->parent()) return;
if (ListeNomsVariables.isEmpty())
    {
    QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Aucune variable n'a encore été définie."));
    return;
    }
AfficherDialog *afficherDialog = new AfficherDialog(this,ListeNomsVariables.join("#"),ListeTypesVariables.join("#"));
if (afficherDialog->exec())
    {
    QString nomvariable=afficherDialog->ui.comboBoxAfficher->currentText();
    QString rang=afficherDialog->ui.lineEditAfficher->text();
    QString code="";
    if (!nomvariable.isEmpty())
	{
	int i=ListeNomsVariables.indexOf(nomvariable);
	if (i>-1)
	    {
	    QString type=ListeTypesVariables.at(i);
	    if (afficherDialog->ui.checkBoxAfficher->isChecked()) code="3#"+nomvariable+"#1";
	    else code="3#"+nomvariable+"#0";
	    if ((type=="LISTE") && (!rang.isEmpty())) 
		{
		curItem->setText(0,QString::fromUtf8("AFFICHER ")+nomvariable+"["+rang+"]");
		code+="#"+rang;
		}
	    else 
		{
		curItem->setText(0,QString::fromUtf8("AFFICHER ")+nomvariable);
		code+="#pasliste";
		}
	    curItem->setData(0,Qt::UserRole,QString(code));
	    }
	ui.treeWidget->setCurrentItem(curItem);
	ActualiserStatut();
	}
    else
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Pas de variable définie"));
	}
    }
InitAfficher();
ActualiserArbre();
}

void MainWindow::AjouterMessage()
{
QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
if (!curItem || !curItem->parent()) return;
MessageDialog *messageDialog = new MessageDialog(this);
if (messageDialog->exec())
    {
    QString message=messageDialog->ui.lineEditMessage->text();
    message.remove("#");
    message.remove("\"");
    message.remove("'");
    QString code="";
    if (!message.isEmpty())
	{
		curItem->setText(0,QString::fromUtf8("AFFICHER \"")+message+"\"");
		if (messageDialog->ui.checkBoxMessage->isChecked()) code="4#"+message+"#1";
		else code="4#"+message+"#0";
		curItem->setData(0,Qt::UserRole,QString(code));
		ui.treeWidget->setCurrentItem(curItem);
		ActualiserStatut();
	}
    else
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Message vide"));
	}
    }
InitMessage();
ActualiserArbre();
}

void MainWindow::AjouterAffectation()
{
QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
if (!curItem || !curItem->parent()) return;
if (ListeNomsVariables.isEmpty())
    {
    QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Aucune variable n'a encore été définie."));
    return;
    }
AffectationDialog *affectationDialog = new AffectationDialog(this,ListeNomsVariables.join("#"),ListeTypesVariables.join("#"));
if (affectationDialog->exec())
    {
    QString nomvariable=affectationDialog->ui.comboBoxAffectation->currentText();
    QString contenu=affectationDialog->ui.lineEditAffectation->text();
    QString rang=affectationDialog->ui.lineEditAffectation2->text();
    contenu.remove("#");
    QString code="";
    if (!nomvariable.isEmpty())
	{
	if (!contenu.isEmpty())
	    {
	    int i=ListeNomsVariables.indexOf(nomvariable);
	    if (i>-1)
		{
		QString type=ListeTypesVariables.at(i);
		code="5#"+nomvariable+"#"+contenu;
		if ((type=="LISTE") && (!rang.isEmpty())) 
		    {
		    curItem->setText(0,nomvariable+"["+rang+"]"+QString::fromUtf8(" PREND_LA_VALEUR ")+contenu);
		    code+="#"+rang;
		    }
		else 
		    {
		    curItem->setText(0,nomvariable+QString::fromUtf8(" PREND_LA_VALEUR ")+contenu);
		    code+="#pasliste";
		    }
		curItem->setData(0,Qt::UserRole,QString(code));
		}
	    ui.treeWidget->setCurrentItem(curItem);
	    ActualiserStatut();
	    }
	else
	    {
	    QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Affectation vide"));
	    }
	}
    else
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Pas de variable définie"));
	}
    }
InitAffectation();
ActualiserArbre();
}

void MainWindow::AjouterCondition()
{
QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
if (!curItem || !curItem->parent()) return;
ConditionDialog *conditionDialog = new ConditionDialog(this);
if (conditionDialog->exec())
    {
    QString condition=conditionDialog->ui.lineEditCondition->text();
    condition.remove("#");
    QString code="";
    QTreeWidgetItem *newItem, *sinonItem;
    if (!condition.isEmpty())
	{
	curItem->setText(0,QString::fromUtf8("SI (")+condition+") "+QString::fromUtf8("ALORS"));
	ui.treeWidget->setItemExpanded (curItem,true);
	code="6#"+condition;
	curItem->setData(0,Qt::UserRole,QString(code));
	newItem = new QTreeWidgetItem(curItem);
	newItem->setText(0,QString::fromUtf8("DEBUT_SI"));
	newItem->setData(0,Qt::UserRole,QString("7#debutsi"));
	newItem = new QTreeWidgetItem(curItem);
	ui.treeWidget->setCurrentItem(newItem);
	newItem = new QTreeWidgetItem(curItem);
	newItem->setText(0,QString::fromUtf8("FIN_SI"));
	newItem->setData(0,Qt::UserRole,QString("8#finsi"));
	if (conditionDialog->ui.checkBoxCondition->isChecked())
	    {
	    sinonItem = new QTreeWidgetItem(curItem,true);
	    sinonItem->setText(0,QString::fromUtf8("SINON"));
	    ui.treeWidget->setItemExpanded (sinonItem,true);
	    code="9#sinon";
	    sinonItem->setData(0,Qt::UserRole,QString(code));
	    newItem = new QTreeWidgetItem(sinonItem);
	    newItem->setText(0,QString::fromUtf8("DEBUT_SINON"));
	    newItem->setData(0,Qt::UserRole,QString("10#debutsinon"));
	    newItem = new QTreeWidgetItem(sinonItem);
	    newItem = new QTreeWidgetItem(sinonItem);
	    newItem->setText(0,QString::fromUtf8("FIN_SINON"));
	    newItem->setData(0,Qt::UserRole,QString("11#finsinon"));
	    }
	ActualiserStatut();
	}
    else
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Pas de condition définie"));
	}
    }
InitCondition();
ActualiserArbre();
}

void MainWindow::AjouterBoucle()
{
QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
if (!curItem || !curItem->parent()) return;
QStringList ListePourVariables;
for (int i = 0; i < ListeNomsVariables.count(); i++) 
    {
    if (ListeTypesVariables.at(i)=="NOMBRE") ListePourVariables.append(ListeNomsVariables.at(i));
    }
PourDialog *pourDialog = new PourDialog(this,ListePourVariables.join("#"));
if (pourDialog->exec())
    {
    QString nomvariable=pourDialog->ui.comboBoxBoucle->currentText();
    QString debut=pourDialog->ui.lineEditBoucle1->text();
    debut.remove("#");
    QString fin=pourDialog->ui.lineEditBoucle2->text();
    fin.remove("#");
    QString code="";
    QTreeWidgetItem *newItem;
    if (!nomvariable.isEmpty())
	{
	if (!debut.isEmpty() && !fin.isEmpty())
	    {
	    int i=ListeNomsVariables.indexOf(nomvariable);
	    if (i>-1)
		{
		curItem->setText(0,QString::fromUtf8("POUR ")+nomvariable+" ALLANT_DE "+debut+ " A "+fin);
		code="12#"+nomvariable+"#"+debut+"#"+fin;
		curItem->setData(0,Qt::UserRole,QString(code));
		newItem = new QTreeWidgetItem(curItem);
		newItem->setText(0,QString::fromUtf8("DEBUT_POUR"));
		newItem->setData(0,Qt::UserRole,QString("13#debutpour"));
		newItem = new QTreeWidgetItem(curItem);
		ui.treeWidget->setCurrentItem(newItem);
		newItem = new QTreeWidgetItem(curItem);
		newItem->setText(0,QString::fromUtf8("FIN_POUR"));
		newItem->setData(0,Qt::UserRole,QString("14#finpour"));
		}
	    ActualiserStatut();
	    }
	else
	    {
	    QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Les valeurs minimales et maximales du compteur ne sont pas définies"));
	    }
	}
    else
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Pas de variable définie"));
	}
    }
InitBoucle();
ActualiserArbre();
}

void MainWindow::AjouterTantque()
{
QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
if (!curItem || !curItem->parent()) return;
TantqueDialog *tantqueDialog = new TantqueDialog(this);
if (tantqueDialog->exec())
    {
    QString condition=tantqueDialog->ui.lineEditTantque->text();
    condition.remove("#");
    QString code="";
    QTreeWidgetItem *newItem;
    if (!condition.isEmpty())
	{
	curItem->setText(0,QString::fromUtf8("TANT_QUE (")+condition+") "+QString::fromUtf8("FAIRE"));
	ui.treeWidget->setItemExpanded (curItem,true);
	code="15#"+condition;
	curItem->setData(0,Qt::UserRole,QString(code));
	newItem = new QTreeWidgetItem(curItem);
	newItem->setText(0,QString::fromUtf8("DEBUT_TANT_QUE"));
	newItem->setData(0,Qt::UserRole,QString("16#debuttantque"));
	newItem = new QTreeWidgetItem(curItem);
	ui.treeWidget->setCurrentItem(newItem);
	newItem = new QTreeWidgetItem(curItem);
	newItem->setText(0,QString::fromUtf8("FIN_TANT_QUE"));
	newItem->setData(0,Qt::UserRole,QString("17#fintantque"));
	ActualiserStatut();
	}
    else
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Pas de condition définie"));
	}
    }
InitTantque();
ActualiserArbre();
}

//***********************************************
void MainWindow::ModifierLire()
{
QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
if (!curItem || !curItem->parent()) return;
if (ListeNomsVariables.isEmpty())
    {
    QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Aucune variable n'a encore été définie."));
    return;
    }

QString excode=curItem->data(0,Qt::UserRole).toString();
QStringList parametres=excode.split("#");
if (parametres.count()==3)
    {
    QString exnomvariable=parametres.at(1);
    int i=ListeNomsVariables.indexOf(exnomvariable);
    QString exrang=parametres.at(2);
    if (i>-1)
	{
	QString extype=ListeTypesVariables.at(i);
	LireDialog *lireDialog = new LireDialog(this,ListeNomsVariables.join("#"),ListeTypesVariables.join("#"));
	int f=lireDialog->ui.comboBoxLire->findText(exnomvariable,Qt::MatchExactly | Qt::MatchCaseSensitive);
	lireDialog->ui.comboBoxLire->setCurrentIndex(f);
	if (extype=="LISTE")
	    {
	    lireDialog->ui.labelLire2->setEnabled(true);
	    lireDialog->ui.lineEditLire->setEnabled(true);
	    lireDialog->ui.lineEditLire->setText(exrang);
	    }
	else
	    {
	    lireDialog->ui.lineEditLire->clear();
	    lireDialog->ui.labelLire2->setEnabled(false);
	    lireDialog->ui.lineEditLire->setEnabled(false);
	    }
	if (lireDialog->exec())
	    {
	    QString nomvariable=lireDialog->ui.comboBoxLire->currentText();
	    QString rang=lireDialog->ui.lineEditLire->text();
	    QString code,type;
	    code="";
	    if (!nomvariable.isEmpty())
		{
		if (curItem && curItem->parent()) 
			{
			int i=ListeNomsVariables.indexOf(nomvariable);
			if (i>-1)
			    {
			    type=ListeTypesVariables.at(i);
			    if ((type=="LISTE") && (!rang.isEmpty())) 
				{
				curItem->setText(0,QString::fromUtf8("LIRE ")+nomvariable+"["+rang+"]");
				code="2#"+nomvariable+"#"+rang;
				}
			    else 
				{
				curItem->setText(0,QString::fromUtf8("LIRE ")+nomvariable);
				code="2#"+nomvariable+"#pasliste";
				}
			    curItem->setData(0,Qt::UserRole,QString(code));
			    }
			ui.treeWidget->setCurrentItem(curItem);
			ActualiserStatut();
			}
		}
	    else
		{
		QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Pas de variable définie"));
		}
	    }
	InitLire();
	ActualiserArbre();
	}
    }
}

void MainWindow::ModifierAfficher()
{
QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
if (!curItem || !curItem->parent()) return;
if (ListeNomsVariables.isEmpty())
    {
    QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Aucune variable n'a encore été définie."));
    return;
    }
QString excode=curItem->data(0,Qt::UserRole).toString();
QStringList parametres=excode.split("#");
if (parametres.count()==4)
    {
    QString exnomvariable=parametres.at(1);
    QString exretourligne=parametres.at(2);
    QString exrang=parametres.at(3);
    int i=ListeNomsVariables.indexOf(exnomvariable);
    if (i>-1)
	{
	QString extype=ListeTypesVariables.at(i);
	AfficherDialog *afficherDialog = new AfficherDialog(this,ListeNomsVariables.join("#"),ListeTypesVariables.join("#"));
	int f=afficherDialog->ui.comboBoxAfficher->findText(exnomvariable,Qt::MatchExactly | Qt::MatchCaseSensitive);
	afficherDialog->ui.comboBoxAfficher->setCurrentIndex(f);
	if (extype=="LISTE")
	    {
	    afficherDialog->ui.labelAfficher2->setEnabled(true);
	    afficherDialog->ui.lineEditAfficher->setEnabled(true);
	    afficherDialog->ui.lineEditAfficher->setText(exrang);
	    }
	else
	    {
	    afficherDialog->ui.lineEditAfficher->clear();
	    afficherDialog->ui.labelAfficher2->setEnabled(false);
	    afficherDialog->ui.lineEditAfficher->setEnabled(false);
	    }
	if (exretourligne=="1") afficherDialog->ui.checkBoxAfficher->setChecked(true);
	else afficherDialog->ui.checkBoxAfficher->setChecked(false);
	if (afficherDialog->exec())
	    {
	    QString nomvariable=afficherDialog->ui.comboBoxAfficher->currentText();
	    QString rang=afficherDialog->ui.lineEditAfficher->text();
	    QString code="";
	    if (!nomvariable.isEmpty())
		{
		int i=ListeNomsVariables.indexOf(nomvariable);
		if (i>-1)
		    {
		    QString type=ListeTypesVariables.at(i);
		    if (afficherDialog->ui.checkBoxAfficher->isChecked()) code="3#"+nomvariable+"#1";
		    else code="3#"+nomvariable+"#0";
		    if ((type=="LISTE") && (!rang.isEmpty())) 
			{
			curItem->setText(0,QString::fromUtf8("AFFICHER ")+nomvariable+"["+rang+"]");
			code+="#"+rang;
			}
		    else 
			{
			curItem->setText(0,QString::fromUtf8("AFFICHER ")+nomvariable);
			code+="#pasliste";
			}
		    curItem->setData(0,Qt::UserRole,QString(code));
		    }
		ui.treeWidget->setCurrentItem(curItem);
		ActualiserStatut();
		}
	    else
		{
		QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Pas de variable définie"));
		}
	    }
	InitAfficher();
	ActualiserArbre();
	}
    }
}

void MainWindow::ModifierMessage()
{
QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
if (!curItem || !curItem->parent()) return;
QString excode=curItem->data(0,Qt::UserRole).toString();
QStringList parametres=excode.split("#");
if (parametres.count()==3)
    {
    QString exmessage=parametres.at(1);
    QString exretourligne=parametres.at(2);
    MessageDialog *messageDialog = new MessageDialog(this);
    messageDialog->ui.lineEditMessage->setText(exmessage);
    if (exretourligne=="1") messageDialog->ui.checkBoxMessage->setChecked(true);
    else messageDialog->ui.checkBoxMessage->setChecked(false);
    if (messageDialog->exec())
	{
	QString message=messageDialog->ui.lineEditMessage->text();
	message.remove("#");
	message.remove("\"");
	message.remove("'");
	QString code="";
	if (!message.isEmpty())
	    {
		    curItem->setText(0,QString::fromUtf8("AFFICHER \"")+message+"\"");
		    if (messageDialog->ui.checkBoxMessage->isChecked()) code="4#"+message+"#1";
		    else code="4#"+message+"#0";
		    curItem->setData(0,Qt::UserRole,QString(code));
		    ui.treeWidget->setCurrentItem(curItem);
		    ActualiserStatut();
	    }
	else
	    {
	    QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Message vide"));
	    }
	}
    InitMessage();
    ActualiserArbre();
    }
}

void MainWindow::ModifierAffectation()
{
QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
if (!curItem || !curItem->parent()) return;
if (ListeNomsVariables.isEmpty())
    {
    QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Aucune variable n'a encore été définie."));
    return;
    }
QString excode=curItem->data(0,Qt::UserRole).toString();
QStringList parametres=excode.split("#");
if (parametres.count()==4)
    {
    QString exnomvariable=parametres.at(1);
    QString excontenu=parametres.at(2);
    QString exrang=parametres.at(3);
    int i=ListeNomsVariables.indexOf(exnomvariable);
    if (i>-1)
	{
	QString extype=ListeTypesVariables.at(i);
	AffectationDialog *affectationDialog = new AffectationDialog(this,ListeNomsVariables.join("#"),ListeTypesVariables.join("#"));
	int f=affectationDialog->ui.comboBoxAffectation->findText(exnomvariable,Qt::MatchExactly | Qt::MatchCaseSensitive);
	affectationDialog->ui.comboBoxAffectation->setCurrentIndex(f);
	affectationDialog->ui.lineEditAffectation->setText(excontenu);
	if (extype=="LISTE")
	    {
	    affectationDialog->ui.labelAffectation3->setEnabled(true);
	    affectationDialog->ui.lineEditAffectation2->setEnabled(true);
	    affectationDialog->ui.lineEditAffectation2->setText(exrang);
	    }
	else
	    {
	    affectationDialog->ui.lineEditAffectation2->clear();
	    affectationDialog->ui.lineEditAffectation2->setEnabled(false);
	    affectationDialog->ui.labelAffectation3->setEnabled(false);
	    }
	if (affectationDialog->exec())
	    {
	    QString nomvariable=affectationDialog->ui.comboBoxAffectation->currentText();
	    QString contenu=affectationDialog->ui.lineEditAffectation->text();
	    QString rang=affectationDialog->ui.lineEditAffectation2->text();
	    contenu.remove("#");
	    QString code="";
	    if (!nomvariable.isEmpty())
		{
		int i=ListeNomsVariables.indexOf(nomvariable);
		if (i>-1)
		    {
		    QString type=ListeTypesVariables.at(i);
		    code="5#"+nomvariable+"#"+contenu;
		    if ((type=="LISTE") && (!rang.isEmpty())) 
			{
			curItem->setText(0,nomvariable+"["+rang+"]"+QString::fromUtf8(" PREND_LA_VALEUR ")+contenu);
			code+="#"+rang;
			}
		    else 
			{
			curItem->setText(0,nomvariable+QString::fromUtf8(" PREND_LA_VALEUR ")+contenu);
			code+="#pasliste";
			}
		    curItem->setData(0,Qt::UserRole,QString(code));
		    }
		ui.treeWidget->setCurrentItem(curItem);
		ActualiserStatut();
		}
	    else
		{
		QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Pas de variable définie"));
		}
	    }
	InitAffectation();
	ActualiserArbre();

	}
    }
}


void MainWindow::ModifierCondition()
{
QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
if (!curItem || !curItem->parent()) return;
QString excode=curItem->data(0,Qt::UserRole).toString();
QStringList parametres=excode.split("#");
if (parametres.count()==2)
    {
    QString excondition=parametres.at(1);
    ConditionDialog *conditionDialog = new ConditionDialog(this);
    conditionDialog->ui.lineEditCondition->setText(excondition);
    conditionDialog->ui.checkBoxCondition->hide();
    if (conditionDialog->exec())
	{
	QString condition=conditionDialog->ui.lineEditCondition->text();
	condition.remove("#");
	QString code="";
	if (!condition.isEmpty())
	    {
	    curItem->setText(0,QString::fromUtf8("SI (")+condition+") "+QString::fromUtf8("ALORS"));
	    ui.treeWidget->setItemExpanded (curItem,true);
	    code="6#"+condition;
	    curItem->setData(0,Qt::UserRole,QString(code));
	    ui.treeWidget->setCurrentItem(curItem);
	    ActualiserStatut();
	    }
	else
	    {
	    QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Pas de condition définie"));
	    }
	}
    InitCondition();
    ActualiserArbre();
    }
}

void MainWindow::ModifierBoucle()
{
QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
if (!curItem || !curItem->parent()) return;
QString excode=curItem->data(0,Qt::UserRole).toString();
QStringList parametres=excode.split("#");
if (parametres.count()==4)
    {
    QString exnomvariable=parametres.at(1);
    QString exdebut=parametres.at(2);
    QString exfin=parametres.at(3);
    QStringList ListePourVariables;
    for (int i = 0; i < ListeNomsVariables.count(); i++) 
	{
	if (ListeTypesVariables.at(i)=="NOMBRE") ListePourVariables.append(ListeNomsVariables.at(i));
	}
    PourDialog *pourDialog = new PourDialog(this,ListePourVariables.join("#"));
    int f=pourDialog->ui.comboBoxBoucle->findText(exnomvariable,Qt::MatchExactly | Qt::MatchCaseSensitive);
    pourDialog->ui.comboBoxBoucle->setCurrentIndex(f);
    pourDialog->ui.lineEditBoucle1->setText(exdebut);
    pourDialog->ui.lineEditBoucle2->setText(exfin);
    if (pourDialog->exec())
	{
	QString nomvariable=pourDialog->ui.comboBoxBoucle->currentText();
	QString debut=pourDialog->ui.lineEditBoucle1->text();
	debut.remove("#");
	QString fin=pourDialog->ui.lineEditBoucle2->text();
	fin.remove("#");
	QString code="";
	if (!nomvariable.isEmpty())
	    {
	    int i=ListeNomsVariables.indexOf(nomvariable);
	    if (i>-1)
		{
		curItem->setText(0,QString::fromUtf8("POUR ")+nomvariable+" ALLANT_DE "+debut+ " A "+fin);
		code="12#"+nomvariable+"#"+debut+"#"+fin;
		curItem->setData(0,Qt::UserRole,QString(code));
		ui.treeWidget->setCurrentItem(curItem);
		}
	    ActualiserStatut();
	    }
	else
	    {
	    QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Pas de variable définie"));
	    }
	}
    InitBoucle();
    ActualiserArbre();
    }
}

void MainWindow::ModifierTantque()
{
QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
if (!curItem || !curItem->parent()) return;
QString excode=curItem->data(0,Qt::UserRole).toString();
QStringList parametres=excode.split("#");
if (parametres.count()==2)
    {
    QString excondition=parametres.at(1);
    TantqueDialog *tantqueDialog = new TantqueDialog(this);
    tantqueDialog->ui.lineEditTantque->setText(excondition);
    if (tantqueDialog->exec())
	{
	QString condition=tantqueDialog->ui.lineEditTantque->text();
	condition.remove("#");
	QString code="";
	if (!condition.isEmpty())
	    {
	    curItem->setText(0,QString::fromUtf8("TANT_QUE (")+condition+") "+QString::fromUtf8("FAIRE"));
	    ui.treeWidget->setItemExpanded (curItem,true);
	    code="15#"+condition;
	    curItem->setData(0,Qt::UserRole,QString(code));
	    ui.treeWidget->setCurrentItem(curItem);
	    ActualiserStatut();
	    }
	else
	    {
	    QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Pas de condition définie"));
	    }
	}
    InitTantque();
    ActualiserArbre();
    }
}

//***********************************************
void MainWindow::LireConfig()
{
QSettings *config=new QSettings(QCoreApplication::applicationDirPath()+"/algobox.ini",QSettings::IniFormat);
config->beginGroup( "algobox" );
dernierRepertoire=config->value("dernier repertoire",QDir::homePath()).toString();
QRect screen = QApplication::desktop()->screenGeometry();
int w= config->value( "Geometries/MainwindowWidth",735).toInt();
int h= config->value( "Geometries/MainwindowHeight",715).toInt() ;
int x= config->value( "Geometries/MainwindowX",10).toInt();
int y= config->value( "Geometries/MainwindowY",10).toInt() ;
resize(w,h);
move(x,y);
windowstate=config->value("MainWindowState").toByteArray();
#ifdef Q_WS_X11
QFontDatabase fdb;
QStringList xf = fdb.families();
QString deft;
if (xf.contains("DejaVu Sans",Qt::CaseInsensitive)) deft="DejaVu Sans";
else if (xf.contains("DejaVu Sans LGC",Qt::CaseInsensitive)) deft="DejaVu Sans LGC";
else if (xf.contains("Bitstream Vera Sans",Qt::CaseInsensitive)) deft="Bitstream Vera Sans";
else if (xf.contains("Luxi Sans",Qt::CaseInsensitive)) deft="Luxi Sans";
else deft=qApp->font().family();
x11fontfamily=config->value("X11/Font Family",deft).toString();
x11fontsize=config->value( "X11/Font Size","10").toInt();

int desktop_env=1; // 1 : no kde ; 2: kde ; 3 : kde4 ; 
QStringList styles = QStyleFactory::keys();
QString kdesession= ::getenv("KDE_FULL_SESSION");
QString kdeversion= ::getenv("KDE_SESSION_VERSION");
if (!kdesession.isEmpty()) desktop_env=2;
if (!kdeversion.isEmpty()) desktop_env=3;
if (desktop_env ==1) //no-kde
    {
    if (styles.contains("GTK+")) QApplication::setStyle(QStyleFactory::create("GTK+"));//gtkstyle
    else QApplication::setStyle(QStyleFactory::create("Cleanlooks"));
    }
else if ((desktop_env ==3) && (styles.contains("Oxygen"))) QApplication::setStyle(QStyleFactory::create("Oxygen")); //kde4+oxygen
else QApplication::setStyle(QStyleFactory::create("Plastique")); //others
QString baseStyleName = qApp->style()->objectName(); //fallback
if (baseStyleName == QLatin1String("Windows")) QApplication::setStyle(QStyleFactory::create("Plastique")); 

QFont x11Font (x11fontfamily,x11fontsize);
QApplication::setFont(x11Font);

QPalette pal = QApplication::palette();
pal.setColor( QPalette::Active, QPalette::Highlight, QColor("#4490d8") );
pal.setColor( QPalette::Inactive, QPalette::Highlight, QColor("#4490d8") );
pal.setColor( QPalette::Disabled, QPalette::Highlight, QColor("#4490d8") );

pal.setColor( QPalette::Active, QPalette::HighlightedText, QColor("#ffffff") );
pal.setColor( QPalette::Inactive, QPalette::HighlightedText, QColor("#ffffff") );
pal.setColor( QPalette::Disabled, QPalette::HighlightedText, QColor("#ffffff") );

pal.setColor( QPalette::Active, QPalette::Base, QColor("#ffffff") );
pal.setColor( QPalette::Inactive, QPalette::Base, QColor("#ffffff") );
pal.setColor( QPalette::Disabled, QPalette::Base, QColor("#ffffff") );

pal.setColor( QPalette::Active, QPalette::WindowText, QColor("#000000") );
pal.setColor( QPalette::Inactive, QPalette::WindowText, QColor("#000000") );
pal.setColor( QPalette::Disabled, QPalette::WindowText, QColor("#000000") );

pal.setColor( QPalette::Active, QPalette::Text, QColor("#000000") );
pal.setColor( QPalette::Inactive, QPalette::Text, QColor("#000000") );
pal.setColor( QPalette::Disabled, QPalette::Text, QColor("#000000") );

pal.setColor( QPalette::Active, QPalette::ButtonText, QColor("#000000") );
pal.setColor( QPalette::Inactive, QPalette::ButtonText, QColor("#000000") );
pal.setColor( QPalette::Disabled, QPalette::ButtonText, QColor("#000000") );

if (desktop_env ==3)
	{
	pal.setColor( QPalette::Active, QPalette::Window, QColor("#eae9e9") );
	pal.setColor( QPalette::Inactive, QPalette::Window, QColor("#eae9e9") );
	pal.setColor( QPalette::Disabled, QPalette::Window, QColor("#eae9e9") );

	pal.setColor( QPalette::Active, QPalette::Button, QColor("#eae9e9") );
	pal.setColor( QPalette::Inactive, QPalette::Button, QColor("#eae9e9") );
	pal.setColor( QPalette::Disabled, QPalette::Button, QColor("#eae9e9") );
	}
else
	{
	pal.setColor( QPalette::Active, QPalette::Window, QColor("#f6f3eb") );
	pal.setColor( QPalette::Inactive, QPalette::Window, QColor("#f6f3eb") );
	pal.setColor( QPalette::Disabled, QPalette::Window, QColor("#f6f3eb") );

	pal.setColor( QPalette::Active, QPalette::Button, QColor("#f6f3eb") );
	pal.setColor( QPalette::Inactive, QPalette::Button, QColor("#f6f3eb") );
	pal.setColor( QPalette::Disabled, QPalette::Button, QColor("#f6f3eb") );
	}

QApplication::setPalette(pal);
#endif
recentFilesList=config->value("Files/Recent Files").toStringList();
config->endGroup();
}

void MainWindow::SauverConfig()
{
QSettings config(QCoreApplication::applicationDirPath()+"/algobox.ini",QSettings::IniFormat);
config.beginGroup( "algobox" );
config.setValue("dernier repertoire",dernierRepertoire);
config.setValue("MainWindowState",saveState(0));
config.setValue("Geometries/MainwindowWidth", width() );
config.setValue("Geometries/MainwindowHeight", height() );
config.setValue("Geometries/MainwindowX", x() );
config.setValue("Geometries/MainwindowY", y() );
#if defined( Q_WS_X11 )
config.setValue("X11/Font Family",x11fontfamily);
config.setValue( "X11/Font Size",x11fontsize);
#endif
if (recentFilesList.count()>0) config.setValue("Files/Recent Files",recentFilesList); 
config.endGroup();
}
//**************************
void MainWindow::InitOuvrir()
{
ui.treeWidget->clear();
ui.treeWidget->setColumnCount(1);
ui.treeWidget->header()->hide();
ui.treeWidget->setEditTriggers(QAbstractItemView::NoEditTriggers);
ui.treeWidget->header()->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);
//ui.treeWidget->header()->setResizeMode(0, QHeaderView::Stretch);
ui.treeWidget->header()->setResizeMode(0, QHeaderView::ResizeToContents);
ui.treeWidget->header()->setStretchLastSection(false);
ui.textEditDescription->clear();
}

void MainWindow::Init()
{
ui.treeWidget->clear();
ui.treeWidget->setColumnCount(1);
ui.treeWidget->header()->hide();
ui.treeWidget->setEditTriggers(QAbstractItemView::NoEditTriggers);
ui.treeWidget->header()->setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);
//ui.treeWidget->header()->setResizeMode(0, QHeaderView::Stretch);
ui.treeWidget->header()->setResizeMode(0, QHeaderView::ResizeToContents);
ui.treeWidget->header()->setStretchLastSection(false);
ui.textEditDescription->clear();

QFont titleFont = qApp->font();
titleFont.setBold(true);

variablesItem =new QTreeWidgetItem(ui.treeWidget);
variablesItem->setText(0,QString("VARIABLES"));
variablesItem->setData(0,Qt::UserRole,QString("100#declarationsvariables"));
variablesItem->setFont(0,titleFont);
ui.treeWidget->setItemExpanded (variablesItem,true);
debutItem =new QTreeWidgetItem(ui.treeWidget);
debutItem->setText(0,QString("DEBUT_ALGORITHME"));
debutItem->setData(0,Qt::UserRole,QString("101#debutalgo"));
debutItem->setFont(0,titleFont);
ui.treeWidget->setItemExpanded (debutItem,true);
finItem =new QTreeWidgetItem(ui.treeWidget);
finItem->setText(0,QString("FIN_ALGORITHME"));
finItem->setData(0,Qt::UserRole,QString("102#finalgo"));
finItem->setFont(0,titleFont);
ui.treeWidget->setCurrentItem(debutItem);
InitVariable();
InitLire();
InitAfficher();
InitMessage();
InitAffectation();
InitCondition();
InitBoucle();
InitTantque();
}

void MainWindow::InitVariable()
{
ui.pushButtonVariable->setEnabled(true);
}
void MainWindow::InitLire()
{
ui.pushButtonVariable->setEnabled(true);

}
void MainWindow::InitAfficher()
{
ui.pushButtonVariable->setEnabled(true);
ActualiserArbre();
}
void MainWindow::InitMessage()
{
ui.pushButtonVariable->setEnabled(true);
ActualiserArbre();
}
void MainWindow::InitAffectation()
{
ui.pushButtonVariable->setEnabled(true);
ActualiserArbre();
}
void MainWindow::InitCondition()
{
ui.pushButtonVariable->setEnabled(true);
ActualiserArbre();
}
void MainWindow::InitBoucle()
{
ui.pushButtonVariable->setEnabled(true);
ActualiserArbre();
}
void MainWindow::InitTantque()
{
ui.pushButtonTantque->setEnabled(true);
ActualiserArbre();
}

void MainWindow::ActiverBoutons()
{
ui.pushButtonLire->setEnabled(true);
ui.pushButtonAfficher->setEnabled(true);
ui.pushButtonMessage->setEnabled(true);
ui.pushButtonAffectation->setEnabled(true);
ui.pushButtonCondition->setEnabled(true);
ui.pushButtonBoucle->setEnabled(true);
ui.pushButtonTantque->setEnabled(true);
}

void MainWindow::DesactiverBoutons()
{
ui.pushButtonLire->setEnabled(false);
ui.pushButtonAfficher->setEnabled(false);
ui.pushButtonMessage->setEnabled(false);
ui.pushButtonAffectation->setEnabled(false);
ui.pushButtonCondition->setEnabled(false);
ui.pushButtonBoucle->setEnabled(false);
ui.pushButtonTantque->setEnabled(false);
}
//********************************
bool MainWindow::NomInterdit(QString nom)
{
bool retour=false;
if ((ListeNomsInterdits.contains(nom,Qt::CaseInsensitive)) || (nom.isEmpty()) || (ListeNomsVariables.contains(nom,Qt::CaseInsensitive))) retour=true;
else
    {
    QStringList motcle=QString("sqrt(,pow(,random(,floor(,cos(,sin(,tan(,exp(,log(,round(,max(,min(").split(",");
    for (int i = 0; i < motcle.count(); i++)
	{
	if (nom.contains(motcle.at(i),Qt::CaseInsensitive) )
	    {
	    retour=true;
	    break;
	    }
	}
    }
return retour;
}
//*******************************
QString MainWindow::GenererCode()
{
QString code=QString::fromUtf8("<!-- Généré par AlgoBox -->\n");
QTextCodec *codec = QTextCodec::codecForName("UTF-8");
QFile modelefile(":/documents/modelepageweb.txt");
modelefile.open(QIODevice::ReadOnly);
QTextStream t(&modelefile);
t.setCodec(codec);
while (!t.atEnd()) 
	{
	code+= t.readLine()+"\n";
	}
modelefile.close();

int nb_branches=ui.treeWidget->topLevelItemCount();
if (nb_branches==0) return code;




QString jscriptcode="";
for (int i = 0; i < nb_branches; i++) jscriptcode+=CodeNoeud(ui.treeWidget->topLevelItem(i));
code.replace("#JAVASCRIPT#",jscriptcode);

QFileInfo fic(nomFichier);
if (nomFichier!="sanstitre") code.replace("#TITRE#","AlgoBox : "+fic.baseName());
else code.replace("#TITRE#","AlgoBox : "+nomFichier);


QString desc=ui.textEditDescription->toPlainText();
if (!desc.isEmpty())
    {
    desc.replace(QString("<"),QString("&lt;"));
    desc.replace(QString(">"),QString("&gt;"));
    desc.replace(QString("\n"),QString("<br>"));
    desc="<div id=\"contentlabel\">Description de l'algorithme :</div>\n<p>\n"+desc+"\n</p>\n";
    code.replace("#DESCRIPTION#",desc);
    }
else code.replace("#DESCRIPTION#","");

QString pseudocode="";
indent=0;
for (int i = 0; i < nb_branches; i++) 
    {
    pseudocode+=AlgoNoeud(ui.treeWidget->topLevelItem(i));
    }
code.replace("#CODE#",pseudocode);
return code;
}

QString MainWindow::CodeVersJavascript(QString algocode)
{
QStringList parametres=algocode.split("#");
QString code="//***************";
if (parametres.count()>0)
  {
  int type_para=parametres.at(0).toInt();
  switch (type_para)
    {
    case 1: //VARIABLES
	    if (parametres.count()==3)
		{
		QString type=parametres.at(1);
		QString nomvariable=parametres.at(2);
		if (type=="NOMBRE") code=nomvariable+"=new Number();";
		else if (type=="CHAINE") code=nomvariable+"=new String();";
		else if (type=="LISTE") code=nomvariable+"=new Array();";
		}
	    break;
    case 2: //LIRE
	    if (parametres.count()==3)
		{
		QString nomvariable=parametres.at(1);
		int i=ListeNomsVariables.indexOf(nomvariable);
		QString rang=parametres.at(2);
		if (i>-1)
		    {
		    QString type=ListeTypesVariables.at(i);
		    if (type=="CHAINE") code=nomvariable+"=prompt(\"Entrer "+nomvariable+"\");";
		    else if (type=="NOMBRE") code=nomvariable+"_temp=prompt(\"Entrer "+nomvariable+"\");\n"+nomvariable+"=parseFloat("+nomvariable+"_temp);";
		    else if (type=="LISTE") code=nomvariable+"_temp=prompt(\"Entrer terme de la liste "+nomvariable+"\");\n"+nomvariable+"["+rang+"]=parseFloat("+nomvariable+"_temp);";
		    }
		}
	    break;
    case 3: //AFFICHER
	    if (parametres.count()==4)
		{
		QString nomvariable=parametres.at(1);
		QString retourligne=parametres.at(2);
		QString rang=parametres.at(3);
		int i=ListeNomsVariables.indexOf(nomvariable);
		if (i>-1)
		    {
		    QString type=ListeTypesVariables.at(i);
		    code="output_content=document.forme1.output.value;\n";
		    if (retourligne=="1") 
			{
			if (type=="LISTE") code+="document.forme1.output.value=output_content+ALGOBOX_FORMAT_TEXTE("+nomvariable+"["+rang+"]"+")+\"\\n\";\n";
			else if (type=="NOMBRE") code+="document.forme1.output.value=output_content+ALGOBOX_FORMAT_TEXTE("+nomvariable+")+\"\\n\";\n";
			else code+="document.forme1.output.value=output_content+"+nomvariable+"+\"\\n\";\n";
			}
		    else 
			{
			if (type=="LISTE") code+="document.forme1.output.value=output_content+ALGOBOX_FORMAT_TEXTE("+nomvariable+"["+rang+"]"+");\n";
			else if (type=="NOMBRE") code+="document.forme1.output.value=output_content+ALGOBOX_FORMAT_TEXTE("+nomvariable+");\n";
			else code+="document.forme1.output.value=output_content+"+nomvariable+";\n";
			}
		    }
		}
	    break;
    case 4: //MESSAGE
	    if (parametres.count()==3)
		{
		QString message=parametres.at(1);
		QString retourligne=parametres.at(2);
		code="output_content=document.forme1.output.value;\n";
		if (retourligne=="1") code+="document.forme1.output.value=output_content+\""+message+"\"+\"\\n\";\n";
		else code+="document.forme1.output.value=output_content+\""+message+"\";\n";
		}
	    break;
    case 5: //AFFECTATION
	    if (parametres.count()==4)
		{
		QString nomvariable=parametres.at(1);
		QString contenu=FiltreCalcul(parametres.at(2));
		QString rang=parametres.at(3);
		int i=ListeNomsVariables.indexOf(nomvariable);
		if (i>-1)
		    {
		    QString type=ListeTypesVariables.at(i);
		    if (type=="LISTE") code=nomvariable+"["+rang+"]="+contenu+";";
		    else code=nomvariable+"="+contenu+";";
		    }
		}
	    break;
    case 6: //SI
	    if (parametres.count()==2)
		{
		QString condition=FiltreCondition(parametres.at(1));
		code="if ("+condition+")";
		}
	    break;
    case 7: //DEBUT_SI
	    if (parametres.count()==2)
		{
		code="{";
		}
	    break;
    case 8: //FIN_SI
	    if (parametres.count()==2)
		{
		code="}";
		}
	    break;
    case 9: //SINON
	    if (parametres.count()==2)
		{
		code="else";
		}
	    break;
    case 10: //DEBUT_SINON
	    if (parametres.count()==2)
		{
		code="{";
		}
	    break;
    case 11: //FIN_SINON
	    if (parametres.count()==2)
		{
		code="}";
		}
	    break;
    case 12: //POUR
	    if (parametres.count()==4)
		{
		QString nomvariable=parametres.at(1);
		QString debut=parametres.at(2);
		QString fin=parametres.at(3);
		code="for ("+nomvariable+"="+debut+";"+nomvariable+"<="+fin+";"+nomvariable+"++)";
		}
	    break;
    case 13: //DEBUT_POUR
	    if (parametres.count()==2)
		{
		code="{";
		}
	    break;
    case 14: //FIN_POUR
	    if (parametres.count()==2)
		{
		code="}";
		}
	    break;
    case 15: //TANT_QUE
	    if (parametres.count()==2)
		{
		QString condition=FiltreCondition(parametres.at(1));
		code="while ("+condition+")";
		}
	    break;
    case 16: //DEBUT_TANT_QUE
	    if (parametres.count()==2)
		{
		code="{";
		}
	    break;
    case 17: //FIN_TANT_QUE
	    if (parametres.count()==2)
		{
		code="}";
		}
	    break;
    case 100: //DECLARATIONS VARIABLES
	    if (parametres.count()==2)
		{
		code="//VARIABLES";
		}
	    break;
    case 101: //DEBUT_ALGO
	    if (parametres.count()==2)
		{
		code="function ALGO()\n{\n";
		code+="output_content=document.forme1.output.value;\n";
		code+="document.forme1.output.value=output_content+\""+QString::fromUtf8("***Algorithme lancé***")+"\"+\"\\n\";\n";
		}
	    break;
    case 102: //FIN_ALGO
	    if (parametres.count()==2)
		{
		code="output_content=document.forme1.output.value;\n";
		code+="document.forme1.output.value=output_content+\"\\n\"+\""+QString::fromUtf8("***Algorithme terminé***")+"\"+\"\\n\";\n";
		code+="}";
		}
	    break;
    case 103: //autres
    default:
	    if (parametres.count()==2)
		{
		code="//***************";
		}
	    break;
    }
  }
return code;
}

QString MainWindow::FiltreNomVariable(QString orig)
{
QStringList motcle=QString("*,+,-,/,%,(,),{,},[,],;,:,!,',\",&,|,!,:,^").split(",");
QString result=orig.trimmed();
for (int i = 0; i < motcle.count(); i++)
    {
    result=result.remove(motcle.at(i));
    }
result.remove(",");
result.replace(" ","_");
return result;
}

QString MainWindow::FiltreCalcul(QString orig)
{
QStringList motcle=QString("sqrt(,pow(,random(,floor(,cos(,sin(,tan(,exp(,log(,round(,max(,min(").split(",");
QString result=orig;
for (int i = 0; i < motcle.count(); i++)
    {
    result=result.replace(motcle.at(i),"Math."+motcle.at(i),Qt::CaseInsensitive);
    }
return result;
}

QString MainWindow::FiltreCondition(QString orig)
{
QString result=orig;
result.replace(" OU "," || ",Qt::CaseInsensitive);
result.replace(" ET "," && ",Qt::CaseInsensitive);
return result;
}

QString MainWindow::CodeNoeud(QTreeWidgetItem *item)
{
QString code="";
if (!item) return code;
code=CodeVersJavascript(item->data(0,Qt::UserRole).toString())+"\n";
int nb_branches=item->childCount();
if (nb_branches>0)
	{
	for (int i = 0; i < nb_branches; i++) code+=CodeNoeud(item->child(i));
	}
return code;
}

QString MainWindow::AlgoNoeud(QTreeWidgetItem *item)
{
QString code="";
for (int i = 0; i < indent; i++) code+="&nbsp;&nbsp;";
if (!item) return code;
QString ligne=item->text(0);
ligne.replace(QString("<"),QString("&lt;"));
ligne.replace(QString(">"),QString("&gt;"));
code+=ligne+"<br>\n";
int nb_branches=item->childCount();
if (nb_branches>0)
	{
	indent+=1;
	for (int i = 0; i < nb_branches; i++) code+=AlgoNoeud(item->child(i));
	indent-=1;
	}
return code;
}

QString MainWindow::AlgoNoeudTexte(QTreeWidgetItem *item)
{
QString code="";
for (int i = 0; i < indent; i++) code+="  ";
if (!item) return code;
code+=item->text(0)+"\n";
int nb_branches=item->childCount();
if (nb_branches>0)
	{
	indent+=1;
	for (int i = 0; i < nb_branches; i++) code+=AlgoNoeudTexte(item->child(i));
	indent-=1;
	}
return code;
}

void MainWindow::JavascriptExport()
{
QTextCodec *codec = QTextCodec::codecForName("UTF-8");
// #if defined( Q_WS_X11 )
// QString htmlfichier=QCoreApplication::applicationDirPath()+"/algo.html";
// #endif
// #if defined( Q_WS_MACX )
// QString htmlfichier=QCoreApplication::applicationDirPath()+"/../Resources/algo.html";
// #endif
// #if defined(Q_WS_WIN)
// QString htmlfichier=QCoreApplication::applicationDirPath()+"/algo.html";
// #endif
QString htmlfichier=QDir::tempPath()+"/algobox_temp.html";
QFile fichier;
fichier.setFileName(htmlfichier);
fichier.open(QIODevice::WriteOnly);
QTextStream out (&fichier);
out.setCodec(codec);
out << GenererCode();
fichier.close();
QFileInfo fic(htmlfichier);
if (fic.exists() && fic.isReadable() )
	{
	BrowserDialog *browserDialog = new BrowserDialog(this,htmlfichier);
	browserDialog->exec();
//	QDesktopServices::openUrl("file:///"+htmlfichier);
	}
}

void MainWindow::ExporterVersTexte()
{
QTextCodec *codec = QTextCodec::codecForName("UTF-8");
QString texte=nomFichier+"  -  "; 
QDate CurrDate = QDate::currentDate();
texte+=CurrDate.toString("dd.MM.yyyy");
texte+="\n\n";
texte+="******************************************\n";
texte+=ui.textEditDescription->toPlainText()+"\n";
texte+="******************************************\n\n";
QString code="";
int nb_branches=ui.treeWidget->topLevelItemCount();
if (nb_branches>0)
    {
    indent=0;
    for (int i = 0; i < nb_branches; i++) 
	{
	code+=AlgoNoeudTexte(ui.treeWidget->topLevelItem(i));
	}
    }
QTextStream t(&code,QIODevice::ReadOnly);
t.setCodec(codec);
int ligne=1;
QString num="";
QString blanc="    ";
while (!t.atEnd()) 
	{
	num=QString::number(ligne);
	num+=blanc.left(blanc.length()-num.length());
	texte+=num+t.readLine()+"\n";
	ligne++;
	}	
QString exportFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Fichier texte (*.txt)"));
if (!exportFichier.isEmpty()) 
    {
    QFileInfo fi(exportFichier);
    dernierRepertoire=fi.absolutePath();
    QFile fichier;
    fichier.setFileName(exportFichier);
    fichier.open(QIODevice::WriteOnly);
    QTextStream out (&fichier);
    out.setCodec(codec);
    out << texte;
    fichier.close();
    }
}

void MainWindow::ExporterVersLatex()
{
QString texte="";
QTextCodec *codec = QTextCodec::codecForName("UTF-8");
QFile texfile(":/documents/modelelatex.txt");
texfile.open(QIODevice::ReadOnly);
QTextStream t(&texfile);
t.setCodec(codec);
while (!t.atEnd()) 
	{
	texte+= t.readLine()+"\n";
	}
texfile.close();
QDate CurrDate = QDate::currentDate();
QString date=CurrDate.toString("dd.MM.yyyy");
QString code="";
int nb_branches=ui.treeWidget->topLevelItemCount();
if (nb_branches>0)
    {
    indent=0;
    for (int i = 0; i < nb_branches; i++) 
	{
	code+=AlgoNoeudTexte(ui.treeWidget->topLevelItem(i));
	}
    }
texte.replace("#TITRE#",nomFichier);
texte.replace("#DATE#",date);
texte.replace("#DESCRIPTION#",ui.textEditDescription->toPlainText());
texte.replace("#CODE#",code);
QString exportFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Document LaTeX (*.tex)"));
if (!exportFichier.isEmpty()) 
    {
    QFileInfo fi(exportFichier);
    dernierRepertoire=fi.absolutePath();
    QFile fichier;
    fichier.setFileName(exportFichier);
    fichier.open(QIODevice::WriteOnly);
    QTextStream out (&fichier);
    out.setCodec(codec);
    out << texte;
    fichier.close();
    }
}

void MainWindow::ExporterVersHtml()
{
QString htmlfichier=QDir::tempPath()+"/algobox_temp.html";
QFileInfo fic(htmlfichier);
if (fic.exists() && fic.isReadable() )
	{
	QString exportFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Page web (*.html)"));
	if (!exportFichier.isEmpty()) 
	    {
	    QFileInfo fi(exportFichier);
	    dernierRepertoire=fi.absolutePath();
	    CopierFichier(htmlfichier,exportFichier);
	    }
	}
}

void MainWindow::Imprimer()
{
QTextCodec *codec = QTextCodec::codecForName("UTF-8");
QString texte=nomFichier+"  -  "; 
QDate CurrDate = QDate::currentDate();
texte+=CurrDate.toString("dd.MM.yyyy");
texte+="\n\n";
texte+="******************************************\n";
texte+=ui.textEditDescription->toPlainText()+"\n";
texte+="******************************************\n\n";
QString code="";
int nb_branches=ui.treeWidget->topLevelItemCount();
if (nb_branches>0)
    {
    indent=0;
    for (int i = 0; i < nb_branches; i++) 
	{
	code+=AlgoNoeudTexte(ui.treeWidget->topLevelItem(i));
	}
    }
QTextStream t(&code,QIODevice::ReadOnly);
t.setCodec(codec);
int ligne=1;
QString num="";
QString blanc="    ";
while (!t.atEnd()) 
	{
	num=QString::number(ligne);
	num+=blanc.left(blanc.length()-num.length());
	texte+=num+t.readLine()+"\n";
	ligne++;
	}	

QPrinter printer;
QPrintDialog *dlg = new QPrintDialog(&printer, this);
if (dlg->exec() != QDialog::Accepted) return;
QTextDocument *document =new QTextDocument(texte);
QFontDatabase fdb;
QStringList xf = fdb.families();
QString deft;
if (xf.contains("DejaVu Sans Mono",Qt::CaseInsensitive)) deft="DejaVu Sans Mono";
#if defined( Q_WS_MACX )
else if (xf.contains("Courier",Qt::CaseInsensitive)) deft="Courier";
#endif
#if defined(Q_WS_WIN)
else if (xf.contains("Courier New",Qt::CaseInsensitive)) deft="Courier New";
#endif
else deft=qApp->font().family();
QFont printFont (deft,8);
document->setDefaultFont(printFont);
document->print(&printer);
}

//********************************
void MainWindow::NouvelAlgo()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("AlgoBox : "),
					QString::fromUtf8("L'algorithme courant a été modifié.\nVoulez-vous l'enregistrer avant d'en créer un nouveau?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverAlgo();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("AlgoBox : ")+nomFichier);
}

void MainWindow::ItemVersXml(QTreeWidgetItem *item,QDomDocument doc ,QDomElement parent)
{
QDomElement element;
element=doc.createElement("item");
element.setAttribute("algoitem",item->text(0));
element.setAttribute("code",item->data(0,Qt::UserRole).toString());
parent.appendChild(element);
int nb_branches=item->childCount();
if (nb_branches>0)
	{
	for (int i = 0; i < nb_branches; i++) 
		{
		ItemVersXml(item->child(i),doc,element);
		}
	}
}

void MainWindow::SauverAlgo()
{
if (nomFichier=="sanstitre" ) SauverSousAlgo();
else
    {
    QFile fichier(nomFichier);
    if (!fichier.open(QIODevice::WriteOnly))
	    {
	    QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible d'enregistrer le fichier"));
	    return;
	    }
    int nb_branches=ui.treeWidget->topLevelItemCount();
    if (nb_branches==0) return ;
    QDomDocument doc;
    QDomProcessingInstruction instr =  doc.createProcessingInstruction("xml","version=\"1.0\" encoding=\"UTF-8\"");
    doc.appendChild(instr);
    QDomElement root=doc.createElement("Algo");
    doc.appendChild(root);
    QDomElement element;
    element=doc.createElement("description");
    element.setAttribute("texte",ui.textEditDescription->toPlainText());
    QTreeWidgetItem *curItem = ui.treeWidget->currentItem();
    if (curItem) element.setAttribute("courant",curItem->text(0));
    else element.setAttribute("courant","");
    root.appendChild(element);
    for (int i = 0; i < nb_branches; i++) ItemVersXml(ui.treeWidget->topLevelItem(i),doc,root);
    QTextStream out (&fichier);
    doc.save(out,4);
    fichier.close();
    QFileInfo fi(nomFichier);
    dernierRepertoire=fi.absolutePath();
    estModifie=false;
    AddRecentFile(nomFichier);
    }
setWindowTitle(QString::fromUtf8("AlgoBox : ")+nomFichier);
}

void MainWindow::SauverSousAlgo()
{
QString nouveauFichier;
if (nomFichier=="sanstitre" ) 
	{
	nouveauFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Algorithme (*.alg)"));
	}
else 
	{
	nouveauFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),nomFichier,QString::fromUtf8("Algorithme (*.alg)"));
	}
if (!nouveauFichier.isEmpty()) 
    {
    nomFichier=nouveauFichier;
    SauverAlgo();
    }
// else
//     {
//     QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible d'enregistrer le fichier"));
//     }
setWindowTitle(QString::fromUtf8("AlgoBox : ")+nomFichier);
}

void MainWindow::XmlVersItem(QTreeWidgetItem *parentItem, QDomElement element)
{
QFont titleFont = qApp->font();
titleFont.setBold(true);

if (!element.hasAttribute("algoitem") || !element.hasAttribute("code")) return;
QTreeWidgetItem *newItem;
if (parentItem) newItem = new QTreeWidgetItem(parentItem);
else newItem = new QTreeWidgetItem(ui.treeWidget);
newItem->setText(0,element.attribute("algoitem"));
newItem->setData(0,Qt::UserRole,QString(element.attribute("code")));
if (newItem->text(0)=="VARIABLES")
    {
    variablesItem=newItem;
    variablesItem->setFont(0,titleFont);
    ui.treeWidget->setItemExpanded (variablesItem,true);
    }
else if (newItem->text(0)=="DEBUT_ALGORITHME")
    {
    debutItem=newItem;
    debutItem->setFont(0,titleFont);
    ui.treeWidget->setItemExpanded (debutItem,true);
    }
else if (newItem->text(0)=="FIN_ALGORITHME")
    {
    finItem=newItem;
    finItem->setFont(0,titleFont);
    }
ui.treeWidget->setCurrentItem(newItem);
ActualiserArbre();
QDomElement enfant = element.firstChildElement();
while (!enfant.isNull()) 
	{
	XmlVersItem(newItem,enfant);
	enfant = enfant.nextSiblingElement();
	}
}

void MainWindow::Ouvrir(QString nouveauFichier)
{
QDomDocument doc;
QFileInfo fi(nouveauFichier);
if (!fi.exists()) return;
dernierRepertoire=fi.absolutePath();
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::ReadOnly)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	return;
	}
if (!doc.setContent(&fichier)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	fichier.close();
	return;
	}
QDomElement root = doc.documentElement();
if (root.tagName() != "Algo")
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Ce fichier ne correspond pas à un algorithme"));
	fichier.close();
	return;
	}
nomFichier=nouveauFichier;
InitOuvrir();

QString courant="";
QDomElement enfant = root.firstChildElement();
while (!enfant.isNull()) 
	{
	if (enfant.hasAttribute("texte") && enfant.hasAttribute("courant"))
	    {
	    ui.textEditDescription->setPlainText(enfant.attribute("texte"));
	    courant=enfant.attribute("courant");
	    }
	else XmlVersItem(0,enfant);
	enfant = enfant.nextSiblingElement();
	}
fichier.close();
estModifie=false;
setWindowTitle(QString::fromUtf8("AlgoBox : ")+nomFichier);
AddRecentFile(nouveauFichier);
if (!courant.isEmpty()) 
    {
    QList<QTreeWidgetItem *> listItems=ui.treeWidget->findItems(courant,Qt::MatchRecursive,0);
    if (listItems.count()>0) ui.treeWidget->setCurrentItem(listItems.at(0));
    }
InitVariable();
InitLire();
InitAfficher();
InitMessage();
InitAffectation();
InitCondition();
InitBoucle();
InitTantque();
}

void MainWindow::ChargerAlgo()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"AlgoBox : ",
					QString::fromUtf8("L'algorithme courant a été modifié.\nVoulez-vous l'enregistrer avant de charger un autre algorithme?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverAlgo();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QString nouveauFichier = QFileDialog::getOpenFileName(this,QString::fromUtf8("Ouvrir Fichier"),dernierRepertoire,"Algorithme (*.alg)");
if (nouveauFichier.isEmpty()) return;
Ouvrir(nouveauFichier);
}

void MainWindow::ActualiserStatut()
{
estModifie=true;
setWindowTitle(QString::fromUtf8("AlgoBox ")+QString::fromUtf8("[modifié]")+" : "+nomFichier);
}
//************************
void MainWindow::Aide()
{
#if defined( Q_WS_X11 )
QString docfile=QCoreApplication::applicationDirPath() + "/ressources/aidealgobox.html";
#endif
#if defined( Q_WS_MACX )
QString docfile=QCoreApplication::applicationDirPath() + "/../Resources/aidealgobox.html";
#endif
#if defined(Q_WS_WIN)
QString docfile=QCoreApplication::applicationDirPath() + "/ressources/aidealgobox.html";
#endif
QFileInfo fic(docfile);
if (fic.exists() && fic.isReadable() )
	{
	QDesktopServices::openUrl("file:///"+docfile);
	}
else { QMessageBox::warning( this,"Erreur",QString::fromUtf8("Fichier d'aide non trouvé"));}
}

void MainWindow::APropos()
{
AproposDialog *abDlg = new AproposDialog(this);
abDlg->exec();
}

void MainWindow::CopierFichier(QString origine,QString destination)
{
if (destination.isEmpty() || origine.isEmpty()) return;
QFileInfo fi_or(origine);
if (!fi_or.exists()) return;
QFile fichier_or(origine);
QFileInfo fi_dest(destination);
if (fi_dest.exists())
	{
	QFile fichier_dest(destination);
	fichier_dest.remove();
	fichier_or.copy(destination);
	}
else
	{
	fichier_or.copy(destination);
	}
}

void MainWindow::SetInterfaceFont()
{
#if defined( Q_WS_X11 )
X11FontDialog *xfdlg = new X11FontDialog(this);
int ft=xfdlg->ui.comboBoxFont->findText (x11fontfamily , Qt::MatchExactly);
xfdlg->ui.comboBoxFont->setCurrentIndex(ft);
xfdlg->ui.spinBoxSize->setValue(x11fontsize);
if (xfdlg->exec())
	{
	x11fontfamily=xfdlg->ui.comboBoxFont->currentText();
	x11fontsize=xfdlg->ui.spinBoxSize->value();
	QFont x11Font (x11fontfamily,x11fontsize);
	QApplication::setFont(x11Font);
	}
#endif
}

void MainWindow::fileOpenRecent()
{
QAction *action = qobject_cast<QAction *>(sender());
if (action) 
    {
    if (estModifie) 
	    {
	    switch(  QMessageBox::warning(this,"AlgoBox : ",
					    QString::fromUtf8("L'algorithme courant a été modifié.\nVoulez-vous l'enregistrer avant de charger un autre algorithme?"),
					    QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					    0,
					    2 ) )
		    {
		    case 0:
		    SauverAlgo();
		    break;
		    case 1:
		    break;
		    case 2:
		    default:
		    return;
		    break;
		    }
	    }
    QString f=action->data().toString();
    Ouvrir(f);
    }
}

void MainWindow::AddRecentFile(const QString &f)
{
if (recentFilesList.contains(f)) return;

if (recentFilesList.count() < 5) recentFilesList.prepend(f);
else
	{
	recentFilesList.removeLast();
	recentFilesList.prepend(f);
	}
UpdateRecentFile();
}

void MainWindow::UpdateRecentFile()
{
for (int i=0; i < recentFilesList.count(); i++)
	{
        recentFileActs[i]->setText(recentFilesList.at(i));
        recentFileActs[i]->setData(recentFilesList.at(i));
        recentFileActs[i]->setVisible(true);
	}
for (int j = recentFilesList.count(); j < 5; ++j) recentFileActs[j]->setVisible(false);
}

void MainWindow::ChargerExemple()
{
#if defined( Q_WS_X11 )
QString exempleRepertoire=QCoreApplication::applicationDirPath() + "/ressources";
#endif
#if defined( Q_WS_MACX )
QString exempleRepertoire=QCoreApplication::applicationDirPath() + "/../Resources";
#endif
#if defined(Q_WS_WIN)
QString exempleRepertoire=QCoreApplication::applicationDirPath() + "/ressources";
#endif
QString f = QFileDialog::getOpenFileName(this,QString::fromUtf8("Ouvrir Fichier"),exempleRepertoire,"Algorithme (*.alg)");
if (!f.isEmpty()) Ouvrir(f);
}
